"use strict";
var core_1 = require('@angular/core');
var rxjs_1 = require('rxjs');
var slider_engine_1 = require('./slider-engine');
var slide_helper_1 = require('./slide-helper');
var SlideService = (function () {
    function SlideService() {
        this.slideChange = new rxjs_1.ReplaySubject();
        this.config = {
            sensitivity: 40,
            minHeight: 400,
            minWidth: 800
        };
    }
    SlideService.prototype.init = function (initialIndex, config) {
        if (initialIndex === void 0) { initialIndex = 0; }
        if (config) {
            this.config = config;
        }
        this.engine = new (this.checkWhichEngineToUse())();
        this.bootstrapEngine(initialIndex);
        this.startResizeListener();
    };
    SlideService.prototype.scrollToIndex = function (toIndex) {
        this.engine.scrollToIndex(toIndex);
    };
    SlideService.prototype.setPause = function (state) {
        this.engine.setPause(state);
    };
    SlideService.prototype.bootstrapEngine = function (initialIndex) {
        var _this = this;
        this.engine.init(initialIndex, this.config)
            .subscribe(function (event) { return _this.slideChange.next(event); });
    };
    SlideService.prototype.startResizeListener = function () {
        var _this = this;
        rxjs_1.Observable.fromEvent(window, 'resize')
            .debounceTime(300)
            .map(function () { return _this.checkWhichEngineToUse(); })
            .filter(function (newEngine) { return !(_this.engine instanceof newEngine); })
            .subscribe(function (newEngine) {
            var previousIndex = _this.engine.remove();
            delete _this.engine;
            _this.engine = new (newEngine)();
            _this.bootstrapEngine(previousIndex);
        });
    };
    SlideService.prototype.checkWhichEngineToUse = function () {
        if (slide_helper_1.SlideHelper.isTouchDevice() ||
            window.innerHeight < this.config.minHeight ||
            window.innerWidth < this.config.minWidth) {
            return slider_engine_1.ScrollSliderEngine;
        }
        else {
            return slider_engine_1.TranslateSliderEngine;
        }
    };
    SlideService.decorators = [
        { type: core_1.Injectable },
    ];
    /** @nocollapse */
    SlideService.ctorParameters = function () { return []; };
    return SlideService;
}());
exports.SlideService = SlideService;
//# sourceMappingURL=slide.service.js.map