"use strict";
var models_1 = require('../../models');
var slide_helper_1 = require('../../slide-helper');
var slide_observables_1 = require('../slide-observables');
var rxjs_1 = require('rxjs');
var scroll_slider_helper_1 = require('./scroll-slider-helper');
var ScrollSliderEngine = (function () {
    function ScrollSliderEngine() {
        this.scrollObservable = rxjs_1.Observable
            .merge(slide_observables_1.SlideObservables.scrollObs)
            .merge(slide_observables_1.SlideObservables.touchMoveObs)
            .throttleTime(200);
        this.pause = false;
        this.slideChange = new rxjs_1.ReplaySubject();
    }
    ScrollSliderEngine.prototype.init = function (initialIndex) {
        var _this = this;
        this.refreshElements();
        this.currentIndex = initialIndex;
        this.setEndState(initialIndex);
        this.scrollSubscription = this.scrollObservable.subscribe(function () { return _this.handleScroll(); });
        return this.slideChange;
    };
    ScrollSliderEngine.prototype.scrollToIndex = function (toIndex) {
        this.slideChange.next(new models_1.SlideEventStart(this.currentIndex, toIndex));
        this.setEndState(toIndex);
    };
    ScrollSliderEngine.prototype.setEndState = function (toIndex) {
        var _this = this;
        this.refreshElements();
        setTimeout(function () { window.scrollTo(window.scrollX, _this.elements[toIndex].offsetTop); return false; }, 100);
        this.slideChange.next(new models_1.SlideEventEnd(this.currentIndex, toIndex));
        this.currentIndex = toIndex;
    };
    ScrollSliderEngine.prototype.setPause = function (state) {
        this.pause = state;
    };
    ScrollSliderEngine.prototype.remove = function () {
        this.scrollSubscription.unsubscribe();
        return this.currentIndex;
    };
    ScrollSliderEngine.prototype.handleScroll = function () {
        var scrollIsIn = scroll_slider_helper_1.ScrollSliderHelper.whatSlideIsScrollIn(this.elements);
        if (this.currentIndex !== scrollIsIn) {
            this.slideChange.next(new models_1.SlideEventStart(this.currentIndex, scrollIsIn));
            this.slideChange.next(new models_1.SlideEventEnd(this.currentIndex, scrollIsIn));
            this.currentIndex = scrollIsIn;
        }
    };
    ScrollSliderEngine.prototype.refreshElements = function () {
        this.elements = slide_helper_1.SlideHelper.getNg2SlideElements();
    };
    return ScrollSliderEngine;
}());
exports.ScrollSliderEngine = ScrollSliderEngine;
//# sourceMappingURL=scroll-slider-engine.js.map