"use strict";
var rxjs_1 = require('rxjs');
var SlideObservables = (function () {
    function SlideObservables() {
    }
    SlideObservables.handleBrowserEdgecases = function (payload) {
        var browser = navigator.userAgent.toLowerCase();
        // For some reason firefox's deltas are really low, so we bump them up!
        if (browser.includes('firefox')) {
            return {
                event: payload.event,
                deltaX: payload.deltaX * 10,
                deltaY: payload.deltaY * 10
            };
        }
        else {
            return payload;
        }
    };
    SlideObservables.scrollObs = rxjs_1.Observable.merge(rxjs_1.Observable.fromEvent(window, 'wheel'))
        .merge(rxjs_1.Observable.fromEvent(window, 'mousewheel'))
        .merge(rxjs_1.Observable.fromEvent(window, 'DOMMouseScroll'))
        .map(function (event) {
        return {
            event: event,
            deltaY: event.deltaY || 0,
            deltaX: event.deltaX || 0
        };
    });
    SlideObservables.touchMoveObs = rxjs_1.Observable.fromEvent(window, 'touchmove')
        .map(function (event) {
        return {
            event: event,
            deltaY: 0,
            deltaX: 0
        };
    });
    SlideObservables.touchObs = rxjs_1.Observable.fromEvent(window, 'touchstart')
        .zip(rxjs_1.Observable.fromEvent(window, 'touchend'))
        .map(function (payload) {
        var anyPayload = payload;
        // Below, layerY is for iOS and Safari devices,
        // while changesTouches.clientY is for Chrome, Firefox and, IE.
        return {
            event: payload[1],
            deltaY: (anyPayload[0].layerY || anyPayload[0].changedTouches[0].clientY) -
                (anyPayload[1].layerY || anyPayload[1].changedTouches[0].clientY),
            deltaX: (anyPayload[0].layerX || anyPayload[0].changedTouches[0].clientX) -
                (anyPayload[1].layerX || anyPayload[1].changedTouches[0].clientX)
        };
    })
        .filter(function (change) { return Math.abs(change.deltaY) > 50; });
    SlideObservables.keys = { 37: true, 38: true, 39: true, 40: true };
    SlideObservables.keyObs = rxjs_1.Observable.fromEvent(document, 'keydown')
        .filter(function (event) { return SlideObservables.keys[event.keyCode]; })
        .map(function (event) {
        var change;
        if (event.key === 'ArrowDown') {
            change = 999999;
        }
        else if (event.key === 'ArrowUp') {
            change = -999999;
        }
        return {
            event: event,
            deltaY: change,
            deltaX: 0
        };
    });
    SlideObservables.deviceMotionObs = rxjs_1.Observable.fromEvent(window, 'devicemotion')
        .filter(function (event) {
        return event.rotationRate.alpha < -5 ||
            event.rotationRate.alpha > 5;
    })
        .throttleTime(300)
        .map(function (event) {
        return {
            event: event,
            deltaY: -(event.rotationRate.alpha * 5),
            deltaX: 0
        };
    });
    SlideObservables.slideObs = rxjs_1.Observable
        .merge(SlideObservables.scrollObs)
        .merge(SlideObservables.keyObs)
        .map(SlideObservables.handleBrowserEdgecases);
    SlideObservables.normalScrollObs = rxjs_1.Observable
        .merge(SlideObservables.scrollObs)
        .merge(SlideObservables.touchMoveObs)
        .map(SlideObservables.handleBrowserEdgecases);
    return SlideObservables;
}());
exports.SlideObservables = SlideObservables;
//# sourceMappingURL=slide-observables.js.map