"use strict";
var slide_observables_1 = require('../slide-observables');
var translate_slider_animator_1 = require('./translate-slider-animator');
var translate_slider_helper_1 = require('./translate-slider-helper');
var slide_helper_1 = require('../../slide-helper');
var models_1 = require('../../models');
var rxjs_1 = require('rxjs');
var TranslateSliderEngine = (function () {
    function TranslateSliderEngine() {
        var _this = this;
        this.scrollObservable = rxjs_1.Observable
            .merge(slide_observables_1.SlideObservables.scrollObs)
            .merge(slide_observables_1.SlideObservables.keyObs)
            .map(slide_observables_1.SlideObservables.handleBrowserEdgecases)
            .filter(function (payload) { return !_this.pause; })
            .filter(function (payload) { return _this.checkWhereIsScroll(payload); })
            .map(function (payload) { slide_helper_1.SlideHelper.preventDefault(payload.event); return payload; })
            .filter(function (payload) { return _this.checkThreshold(payload); })
            .throttleTime(1000);
        this.pause = false;
        this.slideChange = new rxjs_1.ReplaySubject();
    }
    TranslateSliderEngine.prototype.init = function (initialIndex, config) {
        var _this = this;
        this.refreshElements();
        this.config = config;
        this.elements.map(function (element) { return element.classList.add('slide'); });
        this.setEndState(initialIndex);
        this.currentIndex = initialIndex;
        this.slideChange.next(new models_1.SlideEventEnd(null, initialIndex));
        this.scrollSubscription =
            this.scrollObservable.subscribe(function (payload) { return _this.handleScrollEvent(payload); });
        return this.slideChange;
    };
    TranslateSliderEngine.prototype.scrollToIndex = function (toIndex) {
        var _this = this;
        this.refreshElements();
        this.slideChange.next(new models_1.SlideEventStart(this.currentIndex, toIndex));
        translate_slider_animator_1.TranslateSliderAnimator.animateTranslate(this.elements, this.currentIndex, toIndex, function () {
            _this.setEndState(toIndex);
            _this.slideChange.next(new models_1.SlideEventEnd(_this.currentIndex, toIndex));
            _this.currentIndex = toIndex;
        });
    };
    TranslateSliderEngine.prototype.setEndState = function (currentIndex) {
        this.refreshElements();
        this.elements.map(function (element, index) {
            if (index < currentIndex) {
                element.style.display = 'none';
                element.style.transform = 'translate3D(0, -100%, 0)';
            }
            else if (index > currentIndex) {
                element.style.display = 'none';
                element.style.transform = 'translate3D(0, 100%, 0)';
            }
            else if (index === currentIndex) {
                element.style.display = 'block';
                element.style.transform = 'translate3D(0, 0, 0)';
            }
        });
    };
    TranslateSliderEngine.prototype.setPause = function (state) {
        this.pause = state;
    };
    TranslateSliderEngine.prototype.remove = function () {
        this.refreshElements();
        this.elements.map(function (element) {
            element.classList.remove('slide');
            element.style.display = 'block';
            element.style.transform = 'translate3D(0, 0, 0)';
        });
        this.scrollSubscription.unsubscribe();
        return this.currentIndex;
    };
    TranslateSliderEngine.prototype.handleScrollEvent = function (payload) {
        var toIndex = (payload.deltaY >= 0) ? this.currentIndex + 1 : this.currentIndex - 1;
        if (this.elements.length > toIndex && toIndex >= 0)
            this.scrollToIndex(toIndex);
    };
    TranslateSliderEngine.prototype.refreshElements = function () {
        this.elements = slide_helper_1.SlideHelper.getNg2SlideElements();
    };
    TranslateSliderEngine.prototype.checkWhereIsScroll = function (payload) {
        payload.whereIsScrollInsideSlide =
            translate_slider_helper_1.TranslateSliderHelper.whereIsScrollInsideSlide(this.elements, this.currentIndex);
        return payload.whereIsScrollInsideSlide.top && payload.deltaY < 0 ||
            payload.whereIsScrollInsideSlide.bottom && payload.deltaY > 0;
    };
    TranslateSliderEngine.prototype.checkThreshold = function (payload) {
        return Math.abs(payload.deltaY) > this.config.sensitivity;
    };
    return TranslateSliderEngine;
}());
exports.TranslateSliderEngine = TranslateSliderEngine;
//# sourceMappingURL=translate-slider-engine.js.map